@@ -63,10 +63,10 @@ import time
 import traceback
 import warnings
 from contextlib import contextmanager
-from logging import LogRecord
 from logging.handlers import BaseRotatingHandler
 
 from portalocker import LOCK_EX, lock, unlock
+from concurrent_log_handler.__version__ import __author__, __version__
 
 try:
     import pwd
@@ -94,9 +94,6 @@ try:
 except ImportError:
     gzip = None
 
-__version__ = '0.9.17'
-__author__ = "Preston Landers <planders@gmail.com>"
-# __author__ = "Lowell Alleman"
 __all__ = [
     "ConcurrentRotatingFileHandler",
 ]
@@ -106,16 +103,6 @@ if sys.version_info[0] == 2:
     PY2 = True
 
 
-# Workaround for handleError() in Python 2.7+ where record is written to stderr
-# TODO: unused - probably can delete now.
-class NullLogRecord(LogRecord):
-    def __init__(self, *args, **kw):
-        super(NullLogRecord, self).__init__(*args, **kw)
-
-    def __getattr__(self, attr):
-        return None
-
-
 class ConcurrentRotatingFileHandler(BaseRotatingHandler):
     """
     Handler for logging to a set of files, which switches from one file to the
@@ -137,6 +124,9 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
         :param mode: write mode: defaults to 'a' for text append
         :param maxBytes: rotate the file at this size in bytes
         :param backupCount: number of rotated files to keep before deleting.
+            Avoid setting this very high, probably 20 or less, and prefer setting maxBytes higher.
+            A very large number of rollover files can slow down the rollover enough to cause
+            problems due to the mass file renaming while the main lock is held.
         :param encoding: text encoding for logfile
         :param debug: add extra debug statements to this class (for development)
         :param delay: DEPRECATED: value is ignored
@@ -180,6 +170,7 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
         expected. The same is true if this class is used by one application, but
         the RotatingFileHandler is used by another.
         """
+        # noinspection PyTypeChecker
         self.stream = None
         self.stream_lock = None
         self.owner = owner
@@ -196,6 +187,7 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
         self._debug = debug
         self.use_gzip = True if gzip and use_gzip else False
         self.gzip_buffer = 8096
+        self.maxLockAttempts = 20
 
         if unicode_error_policy not in ('ignore', 'replace', 'strict'):
             unicode_error_policy = 'ignore'
@@ -207,7 +199,7 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
 
         if delay not in (None, True):
             warnings.warn(
-                'parameter delay is now ignored and implied as True, '
+                'concurrent_log_handler parameter `delay` is now ignored and implied as True, '
                 'please remove from your config.',
                 DeprecationWarning)
 
@@ -220,7 +212,9 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
         self.terminator = terminator or "\n"
 
         if owner and os.chown and pwd and grp:
+            # noinspection PyUnresolvedReferences
             self._set_uid = pwd.getpwnam(self.owner[0]).pw_uid
+            # noinspection PyUnresolvedReferences
             self._set_gid = grp.getgrnam(self.owner[1]).gr_gid
 
         self.lockFilename = self.getLockFilename()
@@ -303,6 +297,7 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
                     self.stream.flush()
                     self.stream.close()
             finally:
+                # noinspection PyTypeChecker
                 self.stream = None
 
     def _console_log(self, msg, stack=False):
@@ -408,7 +403,7 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
             return   # already locked... recursive?
         self._open_lockfile()
         if self.stream_lock:
-            for i in range(10):
+            for i in range(self.maxLockAttempts):
                 # noinspection PyBroadException
                 try:
                     lock(self.stream_lock, LOCK_EX)
@@ -417,7 +412,8 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
                 except Exception:
                     continue
             else:
-                raise RuntimeError("Cannot acquire lock after 10 attempts")
+                raise RuntimeError(
+                    "Cannot acquire lock after %s attempts" % (self.maxLockAttempts,))
         else:
             self._console_log("No self.stream_lock to lock", stack=True)
 
@@ -499,11 +495,21 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
         # opens "logfile.3" in notepad); we could test rename each file, but
         # nobody's complained about this being an issue; so the additional
         # code complexity isn't warranted.
-        for i in range(self.backupCount - 1, 0, -1):
+
+        do_renames = []
+        for i in range(1, self.backupCount - 1):
             sfn = "%s.%d" % (self.baseFilename, i)
             dfn = "%s.%d" % (self.baseFilename, i + 1)
             if os.path.exists(sfn + gzip_ext):
-                do_rename(sfn, dfn)
+                do_renames.append((sfn, dfn))
+            else:
+                # Break looking for more rollover files as soon as we can't find one
+                # at the expected name.
+                break
+
+        for (sfn, dfn) in reversed(do_renames):
+            do_rename(sfn, dfn)
+
         dfn = self.baseFilename + ".1"
         do_rename(tmpname, dfn)
 
@@ -513,6 +519,7 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
 
         self._console_log("Rotation completed")
 
+    # noinspection PyUnusedLocal
     def shouldRollover(self, record):
         """
         Determine if rollover should occur.
@@ -554,10 +561,10 @@ class ConcurrentRotatingFileHandler(BaseRotatingHandler):
         return
 
     def _do_chown_and_chmod(self, filename):
-        if self._set_uid and self._set_gid:
+        if self._set_uid is not None and self._set_gid is not None:
             os.chown(filename, self._set_uid, self._set_gid)
 
-        if self.chmod and os.chmod:
+        if self.chmod is not None and os.chmod:
             os.chmod(filename, self.chmod)
 
 
